try:
    from .version import __version__, version_info
except ImportError:
    version_info = (0, 0, 0, "x")
    __version__ = "{}.{}.{}+{}".format(*version_info)

authors = (
    ("Dmitry Orlov", "me@mosquito.su"),
)

# TODO: Use mailing list instead
authors_email = ", ".join(
    str(email) for _, email in authors
)

__license__ = "MIT",
__author__ = ", ".join(
    "{0} <{1}>".format(name, email) for name, email in authors
)

package_info = "RSA helpers for PyJWT"

# It's same persons right now
__maintainer__ = __author__

__all__ = (
    "__author__", "__author__", "__license__",
    "__maintainer__", "__version__", "version_info",
)



import time
from datetime import datetime, timedelta
from operator import add, sub
from typing import TYPE_CHECKING, Any, Callable, Dict, Optional, TypeVar, Union

from jwt import PyJWT
from jwt_rsa.rsa import RSAPrivateKey, RSAPublicKey


if TYPE_CHECKING:
    # pylama:ignore=E0602
    DateType = Union[timedelta, datetime, float, int, ellipsis]
else:
    DateType = Union[timedelta, datetime, float, int, type(Ellipsis)]

R = TypeVar("R")


class JWT:
    __slots__ = (
        "__private_key", "__public_key", "__jwt",
        "__expires", "__nbf_delta", "__algorithm",
    )

    DEFAULT_EXPIRATION = 86400 * 30  # one month
    NBF_DELTA = 20
    ALGORITHMS = tuple({
        "RS256", "RS384", "RS512", "ES256", "ES384",
        "ES521", "ES512", "PS256", "PS384", "PS512",
    })

    def __init__(
        self,
        private_key: Optional[RSAPrivateKey] = None,
        public_key: Optional[RSAPublicKey] = None,
        expires: Optional[int] = None,
        nbf_delta: Optional[int] = None,
        algorithm: str = "RS512",
    ):

        self.__private_key = private_key
        self.__public_key = public_key
        self.__jwt = PyJWT(algorithms=self.ALGORITHMS)
        self.__expires = expires or self.DEFAULT_EXPIRATION
        self.__nbf_delta = nbf_delta or self.NBF_DELTA
        self.__algorithm = algorithm

    def _date_to_timestamp(
        self,
        value: DateType,
        default: Callable[[], R],
        timedelta_func: Callable[[float, float], int] = add,
    ) -> Union[int, float, R]:
        if isinstance(value, timedelta):
            return timedelta_func(time.time(), value.total_seconds())
        elif isinstance(value, datetime):
            return value.timestamp()
        elif isinstance(value, (int, float)):
            return value
        elif value is Ellipsis:
            return default()

        raise ValueError(type(value))

    def encode(
        self,
        expired: DateType = ...,
        nbf: DateType = ...,
        **claims: int,
    ) -> str:
        if not self.__private_key:
            raise RuntimeError("Can't encode without private key")

        claims.update(
            dict(
                exp=int(
                    self._date_to_timestamp(
                        expired,
                        lambda: time.time() + self.__expires,
                    ),
                ),
                nbf=int(
                    self._date_to_timestamp(
                        nbf,
                        lambda: time.time() - self.__nbf_delta,
                        timedelta_func=sub,
                    ),
                ),
            ),
        )

        return self.__jwt.encode(
            claims,
            self.__private_key,
            algorithm=self.__algorithm,
        ).decode()

    def decode(
        self, token: str, verify: bool = True, **kwargs: Any,
    ) -> Dict[str, Any]:
        if not self.__public_key:
            raise RuntimeError("Can't decode without public key")

        return self.__jwt.decode(
            token,
            key=self.__public_key,
            verify=verify,
            algorithms=self.ALGORITHMS,
            **kwargs,
        )


if __name__ == "__main__":
    from jwt_rsa.rsa import generate_rsa

    key, public = generate_rsa(2048)

    jwt = JWT(key, public)

    token = jwt.encode()

    print("Token", token)
    print("Content", jwt.decode(token))



import base64
from collections import namedtuple

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.asymmetric.rsa import \
    RSAPrivateKeyWithSerialization as RSAPrivateKey
from cryptography.hazmat.primitives.asymmetric.rsa import \
    RSAPublicKeyWithSerialization as RSAPublicKey


KeyPair = namedtuple("KeyPair", ("private", "public"))


def generate_rsa(bits: int = 2048) -> KeyPair:
    '''
    Generate an RSA keypair with an exponent of 65537 in PEM format
    param: bits The key length in bits
    Return private key and public key
    '''

    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=bits,
        backend=default_backend(),
    )

    public_key = private_key.public_key()

    return KeyPair(private=private_key, public=public_key)


def load_private_key(data: str) -> RSAPrivateKey:
    return serialization.load_der_private_key(
        base64.b64decode(data), None, default_backend(),
    )


def load_public_key(data: str) -> RSAPublicKey:
    return serialization.load_der_public_key(
        base64.b64decode(data), default_backend(),
    )



import argparse
import json

from .rsa import load_private_key
from .token import JWT


parser = argparse.ArgumentParser()
parser.add_argument(
    "-K", "--private-key", required=True,
    help="Private JWT key", type=load_private_key,
)
parser.add_argument(
    "--expired", help="Token expiration",
    type=float, default=...,
)
parser.add_argument(
    "--nbf", help="Token nbf claim",
    type=float, default=...,
)


def main() -> None:
    arguments = parser.parse_args()
    jwt = JWT(private_key=arguments.private_key)
    print(
        jwt.encode(
            expired=arguments.expired,
            nbf=arguments.nbf,
            **json.loads(input("Paste JSON content here: ")),
        ),
    )


if __name__ == "__main__":
    main()



import json
import logging
import os
import sys

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import padding
from jwt_rsa.rsa import load_private_key, load_public_key


def main() -> None:
    logging.basicConfig(level=logging.INFO)

    logging.info("Awaiting JSON on stdin...")
    data = json.load(sys.stdin)

    public_key = load_public_key(data["public"])
    private_key = load_private_key(data["private"])

    payload = os.urandom(1024 * 16)

    signature = private_key.sign(
        payload,
        padding.PSS(
            mgf=padding.MGF1(hashes.SHA256()),
            salt_length=padding.PSS.MAX_LENGTH,
        ),
        hashes.SHA256(),
    )

    logging.info("Signing OK")

    result = public_key.verify(
        signature,
        payload,
        padding.PSS(
            mgf=padding.MGF1(hashes.SHA256()),
            salt_length=padding.PSS.MAX_LENGTH,
        ),
        hashes.SHA256(),
    )

    assert result is None

    logging.info("Verifying OK")


if __name__ == "__main__":
    main()



import argparse
import base64
import json

from cryptography.hazmat.primitives import serialization

from .rsa import generate_rsa


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("-b", "--bits", dest="bits", type=int, default=2048)
    parser.add_argument("-P", "--pem", dest="pem", action="store_true")

    arguments = parser.parse_args()

    private_key, public_key = generate_rsa(arguments.bits)

    if arguments.pem:
        print(
            private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                encryption_algorithm=serialization.NoEncryption(),
                format=serialization.PrivateFormat.PKCS8,
            ).decode(),
        )
        print(
            public_key.public_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PublicFormat.PKCS1,
            ).decode(),
        )
    else:
        print(
            json.dumps(
                {
                    "private": base64.b64encode(
                        private_key.private_bytes(
                            encoding=serialization.Encoding.DER,
                            encryption_algorithm=serialization.NoEncryption(),
                            format=serialization.PrivateFormat.PKCS8,
                        ),
                    ).decode(),
                    "public": base64.b64encode(
                        public_key.public_bytes(
                            encoding=serialization.Encoding.DER,
                            format=serialization.PublicFormat.PKCS1,
                        ),
                    ).decode(),
                }, indent="\t",
            ),
        )


if __name__ == "__main__":
    main()



import os
from setuptools import setup, find_packages
from importlib.machinery import SourceFileLoader


module_name = 'jwt_rsa'

module = SourceFileLoader(
    module_name,
    os.path.join(module_name, '__init__.py')
).load_module()


def load_requirements(fname):
    """ load requirements from a pip requirements file """
    line_iter = (line.strip() for line in open(fname))
    return [line for line in line_iter if line and not line.startswith("#")]


setup(
    name='pyjwt-rsa',
    version=module.__version__,
    author=module.__author__,
    author_email=module.authors_email,
    license=module.__license__,
    description=module.package_info,
    long_description=open("README.rst").read(),
    platforms="all",
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: Russian',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Security',
    ],
    packages=find_packages(exclude=['tests']),
    install_requires=load_requirements('requirements.txt'),
    extras_require={
        'develop': load_requirements('requirements.dev.txt'),
    },
    entry_points={
        'console_scripts': [
            'jwt-rsa-keygen = {}.keygen:main'.format(module_name),
            'jwt-rsa-verify = {}.verify:main'.format(module_name),
            'jwt-rsa-issue= {}.issue:main'.format(module_name),
        ]
    },
    python_requires=">=3.5, <4",
    url='https://github.com/mosquito/pyjwt-rsa'
)
